//
//  UIView+OSVisualDebugging.m
//  Ocasta Studios
//
//  Created by Chris Birch on 24/08/2012.
//  Copyright (c) 2012 Ocasta Studios. All rights reserved.
//

#import "UIView+OSVisualDebugging.h"

@implementation UIView (OSVisualDebugging)

////////////////////////////////
//Category "local storage"
////////////////////////////////

/**
 * A dictionary containing CGPoints (as NSValues) representing the start point from touchesBegan for each view
 * that we are dealing with
 */
static NSMutableDictionary* touchStartForEachView;

/**
 * Returns a string representation of this instances memory address.
 * this is used to give this category some memory to store important stuff.
 */
-(NSString*)__selfMemoryAddressAsString
{
    NSString* memoryAddress = [[NSString alloc] initWithFormat:@"%p", self];
    return memoryAddress;
}

/**
 * Gets the start point as required by the drag algorithm for this specific instance
 */
-(CGPoint)getStartTouch
{
    //Check whether we have saved any startPoints yet
    if (touchStartForEachView != nil)
    {
        //we have so get this views memory address as a string
        NSString* thisViewKey = [self __selfMemoryAddressAsString];
        
        //we retrieve the startPoint for this view by using the memory address as a key to look it up within the static dictionary
        CGPoint touch = [((NSValue*)[touchStartForEachView objectForKey:thisViewKey]) CGPointValue];
        
        return touch;
    }
    
    return CGPointZero;
}

/**
 * Sets the start point as required by the drag algorithm for this specific instance
 */
-(void)setStartTouch: (CGPoint) touch
{
    if (touchStartForEachView == nil)
        touchStartForEachView = [[NSMutableDictionary alloc] init];
    
    NSString* thisViewKey = [self __selfMemoryAddressAsString];
    [touchStartForEachView setObject: [NSValue valueWithCGPoint:touch] forKey:thisViewKey];
    
}

//////////////////////////////
// button Event handling 
/////////////////////////////



/**
 * If this instance is a UIButton then this will remove any previous event listeners that we
 * registered in didMoveToSuperview.
 */
-(void)__removeTargets
{
    if ([self isKindOfClass:[UIButton class]])
    {
        UIButton* button = (UIButton*)self;
        
        //first remove targets if any
        [button removeTarget:self action:@selector(dragBegan:withEvent:) forControlEvents: UIControlEventTouchDown];
        [button removeTarget:self action:@selector(dragMoving:withEvent:) forControlEvents: UIControlEventTouchDragInside];
        [button removeTarget:self action:@selector(dragEnded:withEvent:) forControlEvents: UIControlEventTouchUpInside | UIControlEventTouchUpOutside];
    }
}

-(void)didMoveToSuperview
{
    //If this instance is a button then we need to add event listeners
    //because UIButton steals the events before we get a chance to listen to them.
    if ([self isKindOfClass:[UIButton class]])
    {
        UIButton* button = (UIButton*)self;
        
        //remove any targets that we previously added
        [self __removeTargets];

        [button addTarget:self action:@selector(dragBegan:withEvent:) forControlEvents: UIControlEventTouchDown];
        [button addTarget:self action:@selector(dragMoving:withEvent:) forControlEvents: UIControlEventTouchDragInside];
        [button addTarget:self action:@selector(dragEnded:withEvent:) forControlEvents: UIControlEventTouchUpInside | UIControlEventTouchUpOutside];
    }
}

-(void)dealloc
{
    //if this instance is a button, then remove any event handling we might have set up
    [self __removeTargets];
    
    if (touchStartForEachView != nil)
    {
        //remove the point related to this instance
        [touchStartForEachView removeObjectForKey:[self __selfMemoryAddressAsString]];
        
        //clean up if this was the last view
        if (touchStartForEachView.count == 0)
            touchStartForEachView = nil;
    }
}


///////////////////////////////////
//For UIButton handling
///////////////////////////////////


- (void)dragBegan:(UIControl *)c withEvent:(UIEvent *)event
{
    [self _touchesBegan:[event allTouches] withEvent:event];
}

- (void)dragMoving:(UIControl *)c withEvent:(UIEvent *)event
{
    [self _touchesMoved:[event allTouches] withEvent:event];
}

- (void)dragEnded:(UIControl *)c withEvent:(UIEvent *)event
{
    [self _touchesEnded:[event allTouches] withEvent:event];
}



///////////////////////////////////
//For normal uiview handling
///////////////////////////////////

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    [self _touchesBegan:touches withEvent:event];
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
    [self _touchesMoved:touches withEvent:event];
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    [self _touchesEnded:touches withEvent:event];
}


/////////////////////////////////////////////////////////////////////
// Shared between normal uiview touch handling and button handling
////////////////////////////////////////////////////////////////////

/**
 * Basically touchesBegan but prefaced with a _ so it can be called by two different functions
 */
- (void)_touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    [self setStartTouch:[[touches anyObject] locationInView:self]];
    
    //change colour so we can see which view we touched
    self.backgroundColor = [self randomColour];
    
}
/**
 * Basically touchesMoved but prefaced with a _ so it can be called by two different functions
 */
- (void)_touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
    CGPoint touchStart = [self getStartTouch];
    CGPoint point = [[touches anyObject] locationInView:self];
    self.center = CGPointMake(self.center.x + point.x - touchStart.x, self.center.y + point.y - touchStart.y);
}

/**
 * Basically touchesEnded but prefaced with a _ so it can be called by two different functions
 */
-(void)_touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    //When touches end we log information about the view and its superview to the console
    [self logVisualDebuggingInfo];
}



/**
 * Returns a string containing information about the current instance and its superview
 */
-(NSString*)visualDebuggingInfo
{
    NSMutableString* info = [[NSMutableString alloc] init];
    [info appendFormat:@"\n\n[[Visual Debugging]]------------------------\n\n",nil];
    [info appendFormat:@"\tTouched View: %@\n", self];
    [info appendFormat:@"\tSuper View: %@\n", self.superview];
    [info appendFormat:@"\n[[/Visual Debugging]]------------------------\n\n",nil];
    return info;
}

/**
 * Logs a string containing information about the current instance and its superview
 */
-(void)logVisualDebuggingInfo
{
    NSLog(@"%@",self.visualDebuggingInfo);
}


/**
* From: http://compileyouidontevenknowyou.blogspot.co.uk/2010/06/random-colors-in-objective-c.html
*/
-(UIColor *)randomColour
{
    CGFloat hue = ( arc4random() % 256 / 256.0 );  //  0.0 to 1.0
    CGFloat saturation = ( arc4random() % 128 / 256.0 ) + 0.5;  //  0.5 to 1.0, away from white
    CGFloat brightness = ( arc4random() % 128 / 256.0 ) + 0.5;
    return [UIColor colorWithHue:hue saturation:saturation brightness:brightness alpha:1];
}



@end
